@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,26 +16,43 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "BasicImageView.h"
-#include "BasicImageView.moc"
+#include "BasicImageView.h.moc"
 #include "ImageTransformation.h"
-#include "ImagePresentation.h"
 #include "Dpm.h"
 #include "Dpi.h"
 
-BasicImageView::BasicImageView(
-	QImage const& image, ImagePixmapUnion const& downscaled_image, Margins const& margins)
+BasicImageView::BasicImageView(QImage const& image)
 :	ImageViewBase(
-		image, downscaled_image,
-		ImagePresentation(QTransform(), QRectF(image.rect())), margins
-	),
-	m_dragHandler(*this),
-	m_zoomHandler(*this)
+		image, QImage(),
+		ImageTransformation(image.rect(), Dpm(image))
+	)
 {
-	rootInteractionHandler().makeLastFollower(m_dragHandler);
-	rootInteractionHandler().makeLastFollower(m_zoomHandler);
 }
 
 BasicImageView::~BasicImageView()
 {
 }
+
+void
+BasicImageView::wheelEvent(QWheelEvent* const event)
+{
+	handleZooming(event);
+}
+
+void
+BasicImageView::mousePressEvent(QMouseEvent* const event)
+{
+	handleImageDragging(event);
+}
+
+void
+BasicImageView::mouseReleaseEvent(QMouseEvent* const event)
+{
+	handleImageDragging(event);
+}
+
+void
+BasicImageView::mouseMoveEvent(QMouseEvent* const event)
+{
+	handleImageDragging(event);
+}
