@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,25 +20,22 @@
 #define BASICIMAGEVIEW_H_
 
 #include "ImageViewBase.h"
-#include "DragHandler.h"
-#include "ZoomHandler.h"
-#include "ImagePixmapUnion.h"
-#include "Margins.h"
-#include <QImage>
 
 class BasicImageView : public ImageViewBase
 {
 	Q_OBJECT
 public:
-	BasicImageView(
-		QImage const& image,
-		ImagePixmapUnion const& downscaled_image = ImagePixmapUnion(),
-        Margins const& margins = Margins(0,0,0,0));
+	BasicImageView(QImage const& image);
 	
 	virtual ~BasicImageView();
-private:
-	DragHandler m_dragHandler;
-	ZoomHandler m_zoomHandler;
+protected:
+	virtual void wheelEvent(QWheelEvent* event);
+	
+	virtual void mousePressEvent(QMouseEvent* event);
+	
+	virtual void mouseReleaseEvent(QMouseEvent* event);
+	
+	virtual void mouseMoveEvent(QMouseEvent* event);
 };
 
 #endif
