@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-	Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,17 +17,14 @@
 */
 
 #include "Despeckle.h"
-#include "TaskStatus.h"
 #include "DebugImages.h"
-#include "Dpi.h"
-#include "FastQueue.h"
 #include "imageproc/BinaryImage.h"
 #include "imageproc/ConnectivityMap.h"
 #include "imageproc/Connectivity.h"
-#include <QtGlobal>
-#include <QImage>
+#include <boost/foreach.hpp>
 #include <QDebug>
 #include <vector>
+#include <queue>
 #include <map>
 #include <limits>
 #include <algorithm>
@@ -54,78 +51,18 @@ namespace
 {
 
 /**
- * We treat vertical distances differently from the horizontal ones.
- * We want horizontal proximity to have greater weight, so we
- * multiply the vertical component distances by VERTICAL_SCALE,
- * so that the distance is not:\n
- * sqrt(dx^2 + dy^2)\n
- * but:\n
- * sqrt(dx^2 + (VERTICAL_SCALE*dy)^2)\n
- * Keep in mind that we actually operate on squared distances,
- * so we don't need to take that square root.
+ * When multiplied by the number of pixels in a connected component,
+ * gives the maximum squared distance to another connected component
+ * we may attach it to.
  */
-static int const VERTICAL_SCALE = 2;
-static int const VERTICAL_SCALE_SQ = VERTICAL_SCALE * VERTICAL_SCALE;
-
-struct Settings
-{
-	/**
-	 * When multiplied by the number of pixels in a connected component,
-	 * gives the minimum size (in terms of the number of pixels) of a connected
-	 * component we may attach it to.
-	 */
-	double minRelativeParentWeight;
-
-	/**
-	 * When multiplied by the number of pixels in a connected component,
-	 * gives the maximum squared distance to another connected component
-	 * we may attach it to.
-	 */
-	uint32_t pixelsToSqDist;
-
-	/**
-	 * Defines the minimum width or height in pixels that will guarantee
-	 * the object won't be removed.
-	 */
-	int bigObjectThreshold;
-
-	static Settings get(Despeckle::Level level, Dpi const& dpi);
-};
-
-Settings
-Settings::get(Despeckle::Level const level, Dpi const& dpi)
-{
-	Settings settings;
-
-	int const min_dpi = std::min(dpi.horizontal(), dpi.vertical());
-	double const dpi_factor = min_dpi / 300.0;
-
-	// To silence compiler's warnings.
-	settings.minRelativeParentWeight = 0;
-	settings.pixelsToSqDist = 0;
-	settings.bigObjectThreshold = 0;
-
-	switch (level) {
-		case Despeckle::CAUTIOUS:
-			settings.minRelativeParentWeight = 0.125 * dpi_factor;
-			settings.pixelsToSqDist = 10.0*10.0;
-			settings.bigObjectThreshold = qRound(7 * dpi_factor);
-			break;
-		case Despeckle::NORMAL:
-			settings.minRelativeParentWeight = 0.175 * dpi_factor;
-			settings.pixelsToSqDist = 6.5*6.5;
-			settings.bigObjectThreshold = qRound(12 * dpi_factor);
-			break;
-		case Despeckle::AGGRESSIVE:
-			settings.minRelativeParentWeight = 0.225 * dpi_factor;
-			settings.pixelsToSqDist = 3.5*3.5;
-			settings.bigObjectThreshold = qRound(17 * dpi_factor);
-			break;
-	}
-
-	return settings;
-}
+static uint32_t const PIXELS_TO_SQDIST = 9;
 
+/**
+ * When multiplied by the number of pixels in a connected component,
+ * gives the minimum size (also the number of pixels) of a connected component
+ * we may attach it to.
+ */
+static double const MIN_RELATIVE_PARENT_WEIGHT = 0.5;
 
 struct Component
 {
@@ -168,30 +105,6 @@ uint32_t const Component::ANCHORED_TO_BIG;
 uint32_t const Component::ANCHORED_TO_SMALL;
 uint32_t const Component::TAG_MASK;
 
-struct BoundingBox
-{
-	int top;
-	int left;
-	int bottom;
-	int right;
-
-	BoundingBox() {
-		top = left = std::numeric_limits<int>::max();
-		bottom = right = std::numeric_limits<int>::min();
-	}
-
-	int width() const { return right - left + 1; }
-
-	int height() const { return bottom - top + 1; }
-
-	void extend(int x, int y) {
-		top = std::min(top, y);
-		left = std::min(left, x);
-		bottom = std::max(bottom, y);
-		right = std::max(right, x);
-	}
-};
-
 struct Vector
 {
 	int16_t x;
@@ -231,7 +144,7 @@ union Distance
 	uint32_t sqdist() const {
 		int const x = vec.x;
 		int const y = vec.y;
-		return static_cast<uint32_t>(x * x + VERTICAL_SCALE_SQ * y * y);
+		return static_cast<uint32_t>(x * x + y * y);
 	}
 };
 
@@ -314,19 +227,19 @@ void updateDistance(
  *        or none of the above.
  */
 void tagSourceComponent(
-	Component& source, Component const& target, uint32_t sqdist, Settings const& settings)
+	Component& source, Component const& target, uint32_t sqdist)
 {
 	if (source.anchoredToBig()) {
 		// No point in setting ANCHORED_TO_SMALL.
 		return;
 	}
 	
-	if (sqdist > source.num_pixels * settings.pixelsToSqDist) {
+	if (sqdist > source.num_pixels * PIXELS_TO_SQDIST) {
 		// Too far.
 		return;
 	}
 	
-	if (target.num_pixels >= settings.minRelativeParentWeight * source.num_pixels) {
+	if (target.num_pixels >= MIN_RELATIVE_PARENT_WEIGHT * source.num_pixels) {
 		source.setAnchoredToBig();
 	} else {
 		source.setAnchoredToSmall();
@@ -339,10 +252,10 @@ void tagSourceComponent(
  * being attached, provided that the one it's attached to is also preserved.
  */
 bool canBeAttachedTo(
-	Component const& comp, Component const& target, uint32_t sqdist, Settings const& settings)
+	Component const& comp, Component const& target, uint32_t sqdist)
 {
-	if (sqdist <= comp.num_pixels * settings.pixelsToSqDist) {
-		if (target.num_pixels >= comp.num_pixels * settings.minRelativeParentWeight) {
+	if (sqdist <= comp.num_pixels * PIXELS_TO_SQDIST) {
+		if (target.num_pixels >= comp.num_pixels * MIN_RELATIVE_PARENT_WEIGHT) {
 			return true;
 		}
 	}
@@ -354,8 +267,7 @@ void voronoi(ConnectivityMap& cmap, std::vector<Distance>& dist)
 	int const width = cmap.size().width() + 2;
 	int const height = cmap.size().height() + 2;
 	
-	assert(dist.empty());
-	dist.resize(width * height, Distance::zero());
+	dist.resize(width * height); // We don't really need to initialize it.
 	
 	std::vector<uint32_t> sqdists(width * 2, 0);
 	uint32_t* prev_sqdist_line = &sqdists[0];
@@ -373,7 +285,7 @@ void voronoi(ConnectivityMap& cmap, std::vector<Distance>& dist)
 	}
 	
 	// Top to bottom scan.
-	for (int y = 1; y < height; ++y) {
+	for (int y = 1; y < height - 1; ++y) {
 		dist_line += width;
 		cmap_line += width;
 		dist_line[0].reset(0);
@@ -384,7 +296,6 @@ void voronoi(ConnectivityMap& cmap, std::vector<Distance>& dist)
 		for (int x = 1; x < width - 1; ++x) {
 			if (cmap_line[x]) {
 				this_sqdist_line[x] = 0;
-				assert(dist_line[x] == Distance::zero());
 				continue;
 			}
 			
@@ -396,7 +307,7 @@ void voronoi(ConnectivityMap& cmap, std::vector<Distance>& dist)
 			// Propagate from top.
 			Distance top_dist = dist_line[x - width];
 			uint32_t sqdist_top = prev_sqdist_line[x];
-			sqdist_top += VERTICAL_SCALE_SQ - 2 * VERTICAL_SCALE_SQ * int(top_dist.vec.y);
+			sqdist_top += 1 - (int(top_dist.vec.y) << 1);
 			
 			if (sqdist_left < sqdist_top) {
 				this_sqdist_line[x] = sqdist_left;
@@ -417,7 +328,7 @@ void voronoi(ConnectivityMap& cmap, std::vector<Distance>& dist)
 			Distance right_dist = dist_line[x + 1];
 			uint32_t sqdist_right = this_sqdist_line[x + 1];
 			sqdist_right += 1 + (int(right_dist.vec.x) << 1);
-
+			
 			if (sqdist_right < this_sqdist_line[x]) {
 				this_sqdist_line[x] = sqdist_right;
 				++right_dist.vec.x;
@@ -429,6 +340,14 @@ void voronoi(ConnectivityMap& cmap, std::vector<Distance>& dist)
 		std::swap(this_sqdist_line, prev_sqdist_line);
 	}
 	
+	dist_line[0].reset(0);
+	prev_sqdist_line[0] = dist_line[0].sqdist();
+	for (int x = 1; x < width; ++x) {
+		dist_line[x].vec.x = dist_line[x - 1].vec.x - 1;
+		prev_sqdist_line[x] = prev_sqdist_line[x - 1]
+				- (int(dist_line[x - 1].vec.x) << 1) + 1;
+	}
+	
 	// Bottom to top scan.
 	for (int y = height - 2; y >= 1; --y) {
 		dist_line -= width;
@@ -447,22 +366,20 @@ void voronoi(ConnectivityMap& cmap, std::vector<Distance>& dist)
 			// Propagate from bottom.
 			Distance bottom_dist = dist_line[x + width];
 			uint32_t sqdist_bottom = prev_sqdist_line[x];
-			sqdist_bottom += VERTICAL_SCALE_SQ + 2 * VERTICAL_SCALE_SQ * int(bottom_dist.vec.y);
-
+			sqdist_bottom += 1 + (int(bottom_dist.vec.y) << 1);
+			
 			this_sqdist_line[x] = dist_line[x].sqdist();
 			
 			if (sqdist_right < this_sqdist_line[x]) {
 				this_sqdist_line[x] = sqdist_right;
 				++right_dist.vec.x;
 				dist_line[x] = right_dist;
-				assert(cmap_line[x] == 0 || cmap_line[x + 1] != 0);
 				cmap_line[x] = cmap_line[x + 1];
 			}
 			if (sqdist_bottom < this_sqdist_line[x]) {
 				this_sqdist_line[x] = sqdist_bottom;
 				++bottom_dist.vec.y;
 				dist_line[x] = bottom_dist;
-				assert(cmap_line[x] == 0 || cmap_line[x + width] != 0);
 				cmap_line[x] = cmap_line[x + width];
 			}
 		}
@@ -478,7 +395,6 @@ void voronoi(ConnectivityMap& cmap, std::vector<Distance>& dist)
 				this_sqdist_line[x] = sqdist_left;
 				--left_dist.vec.x;
 				dist_line[x] = left_dist;
-				assert(cmap_line[x] == 0 || cmap_line[x - 1] != 0);
 				cmap_line[x] = cmap_line[x - 1];
 			}
 		}
@@ -498,14 +414,6 @@ void voronoiSpecial(ConnectivityMap& cmap, std::vector<Distance>& dist, Distance
 	
 	Distance* dist_line = &dist[0];
 	uint32_t* cmap_line = cmap.paddedData();
-
-	dist_line[0].reset(0);
-	prev_sqdist_line[0] = dist_line[0].sqdist();
-	for (int x = 1; x < width; ++x) {
-		dist_line[x].vec.x = dist_line[x - 1].vec.x - 1;
-		prev_sqdist_line[x] = prev_sqdist_line[x - 1]
-				- (int(dist_line[x - 1].vec.x) << 1) + 1;
-	}
 	
 	// Top to bottom scan.
 	for (int y = 1; y < height - 1; ++y) {
@@ -532,7 +440,6 @@ void voronoiSpecial(ConnectivityMap& cmap, std::vector<Distance>& dist, Distance
 					this_sqdist_line[x] = sqdist_left;
 					--left_dist.vec.x;
 					dist_line[x] = left_dist;
-					assert(cmap_line[x] == 0 || cmap_line[x - 1] != 0);
 					cmap_line[x] = cmap_line[x - 1];
 				}
 			}
@@ -541,12 +448,11 @@ void voronoiSpecial(ConnectivityMap& cmap, std::vector<Distance>& dist, Distance
 			Distance top_dist = dist_line[x - width];
 			if (top_dist != special_distance) {
 				uint32_t sqdist_top = prev_sqdist_line[x];
-				sqdist_top += VERTICAL_SCALE_SQ - 2 * VERTICAL_SCALE_SQ * int(top_dist.vec.y);
+				sqdist_top += 1 - (int(top_dist.vec.y) << 1);
 				if (sqdist_top < this_sqdist_line[x]) {
 					this_sqdist_line[x] = sqdist_top;
 					--top_dist.vec.y;
 					dist_line[x] = top_dist;
-					assert(cmap_line[x] == 0 || cmap_line[x - width] != 0);
 					cmap_line[x] = cmap_line[x - width];
 				}
 			}
@@ -567,7 +473,6 @@ void voronoiSpecial(ConnectivityMap& cmap, std::vector<Distance>& dist, Distance
 					this_sqdist_line[x] = sqdist_right;
 					++right_dist.vec.x;
 					dist_line[x] = right_dist;
-					assert(cmap_line[x] == 0 || cmap_line[x + 1] != 0);
 					cmap_line[x] = cmap_line[x + 1];
 				}
 			}
@@ -601,7 +506,6 @@ void voronoiSpecial(ConnectivityMap& cmap, std::vector<Distance>& dist, Distance
 					this_sqdist_line[x] = sqdist_right;
 					++right_dist.vec.x;
 					dist_line[x] = right_dist;
-					assert(cmap_line[x] == 0 || cmap_line[x + 1] != 0);
 					cmap_line[x] = cmap_line[x + 1];
 				}
 			}
@@ -610,12 +514,11 @@ void voronoiSpecial(ConnectivityMap& cmap, std::vector<Distance>& dist, Distance
 			Distance bottom_dist = dist_line[x + width];
 			if (bottom_dist != special_distance) {
 				uint32_t sqdist_bottom = prev_sqdist_line[x];
-				sqdist_bottom += VERTICAL_SCALE_SQ + 2 * VERTICAL_SCALE_SQ * int(bottom_dist.vec.y);
+				sqdist_bottom += 1 + (int(bottom_dist.vec.y) << 1);
 				if (sqdist_bottom < this_sqdist_line[x]) {
 					this_sqdist_line[x] = sqdist_bottom;
 					++bottom_dist.vec.y;
 					dist_line[x] = bottom_dist;
-					assert(cmap_line[x] == 0 || cmap_line[x + width] != 0);
 					cmap_line[x] = cmap_line[x + width];
 				}
 			}
@@ -636,7 +539,6 @@ void voronoiSpecial(ConnectivityMap& cmap, std::vector<Distance>& dist, Distance
 					this_sqdist_line[x] = sqdist_left;
 					--left_dist.vec.x;
 					dist_line[x] = left_dist;
-					assert(cmap_line[x] == 0 || cmap_line[x - 1] != 0);
 					cmap_line[x] = cmap_line[x - 1];
 				}
 			}
@@ -694,80 +596,61 @@ void voronoiDistances(
 } // anonymous namespace
 
 
-BinaryImage
-Despeckle::despeckle(
-	BinaryImage const& src, Dpi const& dpi, Level const level,
-	TaskStatus const& status, DebugImages* const dbg)
+BinaryImage despeckle(
+	BinaryImage const& src, int const big_object_threshold,
+	DebugImages* const dbg)
 {
 	BinaryImage dst(src);
-	despeckleInPlace(dst, dpi, level, status, dbg);
+	despeckleInPlace(dst, big_object_threshold, dbg);
 	return dst;
 }
 
-void
-Despeckle::despeckleInPlace(
-	BinaryImage& image, Dpi const& dpi, Level const level,
-	TaskStatus const& status, DebugImages* const dbg)
+void despeckleInPlace(
+	BinaryImage& image, int const big_object_threshold,
+	DebugImages* const dbg)
 {
-	Settings const settings(Settings::get(level, dpi));
-
 	ConnectivityMap cmap(image, CONN8);
 	if (cmap.maxLabel() == 0) {
 		// Completely white image?
 		return;
 	}
-
-	status.throwIfCancelled();
 	
 	std::vector<Component> components(cmap.maxLabel() + 1);
-	std::vector<BoundingBox> bounding_boxes(cmap.maxLabel() + 1);
-
+	
 	int const width = image.width();
 	int const height = image.height();
-
+	
 	uint32_t* const cmap_data = cmap.data();
 	
-	// Count the number of pixels and a bounding rect of each component.
+	// Count the number of pixels in each component.
 	uint32_t* cmap_line = cmap_data;
 	int const cmap_stride = cmap.stride();
 	for (int y = 0; y < height; ++y) {
 		for (int x = 0; x < width; ++x) {
-			uint32_t const label = cmap_line[x];
-			++components[label].num_pixels;
-			bounding_boxes[label].extend(x, y);
+			++components[cmap_line[x]].num_pixels;
 		}
 		cmap_line += cmap_stride;
 	}
 	
-	status.throwIfCancelled();
-
 	// Unify big components into one.
 	std::vector<uint32_t> remapping_table(components.size());
 	uint32_t unified_big_component = 0;
 	uint32_t next_avail_component = 1;
 	for (uint32_t label = 1; label <= cmap.maxLabel(); ++label) {
-		if (bounding_boxes[label].width() < settings.bigObjectThreshold &&
-				bounding_boxes[label].height() < settings.bigObjectThreshold) {
-			components[next_avail_component] = components[label];
+		if ((int)components[label].num_pixels < big_object_threshold) {
 			remapping_table[label] = next_avail_component;
 			++next_avail_component;
 		} else {
 			if (unified_big_component == 0) {
 				unified_big_component = next_avail_component;
 				++next_avail_component;
-				components[unified_big_component] = components[label];
-				// Set num_pixels to a large value so that canBeAttachedTo()
-				// always allows attaching to any such component.
-				components[unified_big_component].num_pixels = width * height;
 			}
 			remapping_table[label] = unified_big_component;
 		}
+		components[remapping_table[label]] = components[label];
 	}
 	components.resize(next_avail_component);
-	std::vector<BoundingBox>().swap(bounding_boxes); // We don't need them any more.
 	
-	status.throwIfCancelled();
-
 	uint32_t const max_label = next_avail_component - 1;
 	
 	// Remapping individual pixels.
@@ -781,8 +664,6 @@ Despeckle::despeckleInPlace(
 	if (dbg) {
 		dbg->add(cmap.visualized(), "big_components_unified");
 	}
-
-	status.throwIfCancelled();
 	
 	// Build a Voronoi diagram.
 	std::vector<Distance> distance_matrix;
@@ -791,8 +672,6 @@ Despeckle::despeckleInPlace(
 		dbg->add(cmap.visualized(), "voronoi");
 	}
 	
-	status.throwIfCancelled();
-
 	Distance* const distance_data = &distance_matrix[0] + width + 3;
 	
 	// Now build a bidirectional map of distances between neighboring
@@ -803,16 +682,14 @@ Despeckle::despeckleInPlace(
 	
 	voronoiDistances(cmap, distance_matrix, conns);
 	
-	status.throwIfCancelled();
-
 	// Tag connected components with ANCHORED_TO_BIG or ANCHORED_TO_SMALL.
-	for (Connections::value_type const& pair: conns) {
+	BOOST_FOREACH(Connections::value_type const& pair, conns) {
 		Connection const conn(pair.first);
 		uint32_t const sqdist = pair.second;
 		Component& comp1 = components[conn.lesser_label];
 		Component& comp2 = components[conn.greater_label];
-		tagSourceComponent(comp1, comp2, sqdist, settings);
-		tagSourceComponent(comp2, comp1, sqdist, settings);
+		tagSourceComponent(comp1, comp2, sqdist);
+		tagSourceComponent(comp2, comp1, sqdist);
 	}
 	
 	// Prevent it from growing when we compute the Voronoi diagram
@@ -820,14 +697,11 @@ Despeckle::despeckleInPlace(
 	components[unified_big_component].setAnchoredToBig();
 	
 	bool have_anchored_to_small_but_not_big = false;
-	for (Component const& comp: components) {
+	BOOST_FOREACH(Component const& comp, components) {
 		have_anchored_to_small_but_not_big = comp.anchoredToSmallButNotBig();
 	}
 	
 	if (have_anchored_to_small_but_not_big) {
-		
-		status.throwIfCancelled();
-		
 		// Give such components a second chance.  Maybe they do have
 		// big neighbors, but Voronoi regions from a smaller ones
 		// block the path to the bigger ones.
@@ -856,8 +730,6 @@ Despeckle::despeckleInPlace(
 			}
 		}
 		
-		status.throwIfCancelled();
-
 		// Calculate the Voronoi diagram again, but this time
 		// treat pixels with a special distance in such a way
 		// to prevent them from spreading but also preventing
@@ -867,19 +739,15 @@ Despeckle::despeckleInPlace(
 			dbg->add(cmap.visualized(), "voronoi_special");
 		}
 		
-		status.throwIfCancelled();
-
 		// We've got new connections.  Add them to the map.
 		voronoiDistances(cmap, distance_matrix, conns);
 	}
 	
-	status.throwIfCancelled();
-
 	// Clear the distance matrix.
 	std::vector<Distance>().swap(distance_matrix);
 	
 	// Remove tags from components.
-	for (Component& comp: components) {
+	BOOST_FOREACH(Component& comp, components) {
 		comp.clearTags();
 	}
 	
@@ -895,19 +763,17 @@ Despeckle::despeckleInPlace(
 		uint32_t const sqdist = it->second;
 		Component const& comp1 = components[label1];
 		Component const& comp2 = components[label2];
-		if (canBeAttachedTo(comp1, comp2, sqdist, settings)) {
+		if (canBeAttachedTo(comp1, comp2, sqdist)) {
 			target_source.push_back(TargetSourceConn(label2, label1));
 		}
-		if (canBeAttachedTo(comp2, comp1, sqdist, settings)) {
+		if (canBeAttachedTo(comp2, comp1, sqdist)) {
 			target_source.push_back(TargetSourceConn(label1, label2));
 		}
 		conns.erase(it);
 	}
-
+	
 	std::sort(target_source.begin(), target_source.end());
 	
-	status.throwIfCancelled();
-
 	// Create an index for quick access to a group of connections
 	// with a specified target.
 	std::vector<size_t> target_source_idx;
@@ -927,7 +793,7 @@ Despeckle::despeckleInPlace(
 	}
 	
 	// Labels of components that are to be retained.
-	FastQueue<uint32_t> ok_labels;
+	std::queue<uint32_t> ok_labels;
 	ok_labels.push(unified_big_component);
 	
 	while (!ok_labels.empty()) {
@@ -949,8 +815,6 @@ Despeckle::despeckleInPlace(
 		}
 	}
 	
-	status.throwIfCancelled();
-
 	// Remove unmarked components from the binary image.
 	uint32_t const msb = uint32_t(1) << 31;
 	uint32_t* image_line = image.data();
