@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-	Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,10 +20,7 @@
 #include "ImageId.h"
 #include "ImageLoader.h"
 #include "AtomicFileOverwriter.h"
-#include "RelinkablePath.h"
-#include "OutOfMemoryHandler.h"
 #include "imageproc/Scale.h"
-#include "imageproc/GrayImage.h"
 #include <QCoreApplication>
 #include <QCryptographicHash>
 #include <QThread>
@@ -39,19 +36,16 @@
 #include <QEvent>
 #include <QSize>
 #include <QDebug>
-#ifndef Q_MOC_RUN
 #include <boost/multi_index_container.hpp>
 #include <boost/multi_index/ordered_index.hpp>
 #include <boost/multi_index/sequenced_index.hpp>
 #include <boost/multi_index/member.hpp>
-#endif
-#include <algorithm>
+#include <boost/foreach.hpp>
 #include <vector>
-#include <new>
 
 using namespace ::boost;
 using namespace ::boost::multi_index;
-using namespace imageproc;
+
 
 class ThumbnailPixmapCache::Item
 {
@@ -115,8 +109,6 @@ public:
 		int max_cached_pixmaps, int expiration_threshold);
 	
 	~Impl();
-
-	void setThumbDir(QString const& thumb_dir);
 	
 	Status request(
 		ImageId const& image_id, QPixmap& pixmap, bool load_now = false,
@@ -267,7 +259,7 @@ ThumbnailPixmapCache::ThumbnailPixmapCache(
 	int const max_cached_pixmaps, int const expiration_threshold)
 :	m_ptrImpl(
 		new Impl(
-			RelinkablePath::normalize(thumb_dir), max_thumb_size,
+			thumb_dir, max_thumb_size,
 			max_cached_pixmaps, expiration_threshold
 		)
 	)
@@ -278,12 +270,6 @@ ThumbnailPixmapCache::~ThumbnailPixmapCache()
 {
 }
 
-void
-ThumbnailPixmapCache::setThumbDir(QString const& thumb_dir)
-{
-	m_ptrImpl->setThumbDir(RelinkablePath::normalize(thumb_dir));
-}
-
 ThumbnailPixmapCache::Status
 ThumbnailPixmapCache::loadFromCache(ImageId const& image_id, QPixmap& pixmap)
 {
@@ -340,12 +326,7 @@ ThumbnailPixmapCache::Impl::Impl(
 	m_threadStarted(false),
 	m_shuttingDown(false)
 {
-	// Note that QDir::mkdir() will fail if the parent directory,
-	// that is $OUT/cache doesn't exist. We want that behaviour,
-	// as otherwise when loading a project from a different machine,
-	// a whole bunch of bogus directories would be created.
-	QDir().mkdir(m_thumbDir);
-
+	QDir().mkpath(m_thumbDir);
 	m_backgroundLoader.moveToThread(this);
 }
 
@@ -365,26 +346,6 @@ ThumbnailPixmapCache::Impl::~Impl()
 	wait();
 }
 
-void
-ThumbnailPixmapCache::Impl::setThumbDir(QString const& thumb_dir)
-{
-	QMutexLocker locker(&m_mutex);
-
-	if (thumb_dir == m_thumbDir) {
-		return;
-	}
-
-	m_thumbDir = thumb_dir;
-
-	for (Item const& item: m_loadQueue) {
-		// This trick will make all queued tasks to expire.
-		m_totalLoadAttempts = std::max(
-			m_totalLoadAttempts,
-			item.precedingLoadAttempts + m_expirationThreshold + 1
-		);
-	}
-}
-
 ThumbnailPixmapCache::Status
 ThumbnailPixmapCache::Impl::request(
 	ImageId const& image_id, QPixmap& pixmap, bool const load_now,
@@ -515,7 +476,7 @@ ThumbnailPixmapCache::Impl::ensureThumbnailExists(
 	
 	AtomicFileOverwriter overwriter;
 	QIODevice* iodev = overwriter.startWriting(thumb_file_path);
-	if (iodev && thumbnail.save(iodev, "PNG")) {
+	if (thumbnail.save(iodev, "PNG")) {
 		overwriter.commit();
 	}
 }
@@ -544,7 +505,7 @@ ThumbnailPixmapCache::Impl::recreateThumbnail(
 	// Note that we may be called from multiple threads at the same time.
 	AtomicFileOverwriter overwriter;
 	QIODevice* iodev = overwriter.startWriting(thumb_file_path);
-	if (iodev && thumbnail.save(iodev, "PNG")) {
+	if (thumbnail.save(iodev, "PNG")) {
 		thumb_written = overwriter.commit();
 	} else {
 		overwriter.abort();
@@ -599,70 +560,66 @@ ThumbnailPixmapCache::Impl::backgroundProcessing()
 	assert(QCoreApplication::instance()->thread() != QThread::currentThread());
 	
 	for (;;) {
-		try {
-			// We are going to initialize these while holding the mutex.
-			LoadQueue::iterator lq_it;
-			ImageId image_id;
-			QString thumb_dir;
-			QSize max_thumb_size;
-
-			{
-				QMutexLocker const locker(&m_mutex);
-
-				if (m_shuttingDown || m_items.empty()) {
-					break;
-				}
-
-				lq_it = m_loadQueue.begin();
-				image_id = lq_it->imageId;
-
-				if (lq_it->status != Item::QUEUED) {
-					// All QUEUED items precede any other items
-					// in the load queue, so it means there are no
-					// QUEUED items at all.
-					assert(m_numQueuedItems == 0);
-					break;
-				}
-
-				// By marking the item as IN_PROGRESS, we prevent it
-				// from being processed again before the GUI thread
-				// receives our LoadResultEvent.
-				queuedToInProgress(lq_it);
-
-				if (m_totalLoadAttempts - lq_it->precedingLoadAttempts
-						> m_expirationThreshold) {
-
-					// Expire this request.  The reasoning behind
-					// request expiration is described in
-					// ThumbnailLoadResult::REQUEST_EXPIRED
-					// documentation.
-
-					postLoadResult(
-						lq_it, QImage(),
-						ThumbnailLoadResult::REQUEST_EXPIRED
-					);
-					continue;
-				}
-
-				// Expired requests don't count as load attempts.
-				++m_totalLoadAttempts;
-
-				// Copy those while holding the mutex.
-				thumb_dir = m_thumbDir;
-				max_thumb_size = m_maxThumbSize;
-			} // mutex scope
-
-			QImage const image(
-				loadSaveThumbnail(image_id, thumb_dir, max_thumb_size)
-			);
-
-			ThumbnailLoadResult::Status const status = image.isNull()
-				? ThumbnailLoadResult::LOAD_FAILED
-				: ThumbnailLoadResult::LOADED;
-			postLoadResult(lq_it, image, status);
-		} catch (std::bad_alloc const&) {
-			OutOfMemoryHandler::instance().handleOutOfMemorySituation();
-		}
+		// We are going to initialize these while holding the mutex.
+		LoadQueue::iterator lq_it;
+		ImageId image_id;
+		QString thumb_dir;
+		QSize max_thumb_size;
+		
+		{
+			QMutexLocker const locker(&m_mutex);
+			
+			if (m_shuttingDown || m_items.empty()) {
+				break;
+			}
+			
+			lq_it = m_loadQueue.begin();
+			image_id = lq_it->imageId;
+			
+			if (lq_it->status != Item::QUEUED) {
+				// All QUEUED items precede any other items
+				// in the load queue, so it means there are no
+				// QUEUED items at all.
+				assert(m_numQueuedItems == 0);
+				break;
+			}
+			
+			// By marking the item as IN_PROGRESS, we prevent it
+			// from being processed again before the GUI thread
+			// receives our LoadResultEvent.
+			queuedToInProgress(lq_it);
+			
+			if (m_totalLoadAttempts - lq_it->precedingLoadAttempts
+					> m_expirationThreshold) {
+				
+				// Expire this request.  The reasoning behind
+				// request expiration is described in
+				// ThumbnailLoadResult::REQUEST_EXPIRED
+				// documentation.
+				
+				postLoadResult(
+					lq_it, QImage(),
+					ThumbnailLoadResult::REQUEST_EXPIRED
+				);
+				continue;
+			}
+			
+			// Expired requests don't count as load attempts.
+			++m_totalLoadAttempts;
+			
+			// Copy those while holding the mutex.
+			thumb_dir = m_thumbDir;
+			max_thumb_size = m_maxThumbSize;
+		} // mutex scope
+		
+		QImage const image(
+			loadSaveThumbnail(image_id, thumb_dir, max_thumb_size)
+		);
+		
+		ThumbnailLoadResult::Status const status = image.isNull()
+			? ThumbnailLoadResult::LOAD_FAILED
+			: ThumbnailLoadResult::LOADED;
+		postLoadResult(lq_it, image, status);
 	}
 }
 
@@ -678,7 +635,7 @@ ThumbnailPixmapCache::Impl::loadSaveThumbnail(
 		return image;
 	}
 	
-	image = ImageLoader::load(image_id);
+	image = ImageLoader::load(image_id.filePath(), image_id.page());
 	if (image.isNull()) {
 		return QImage();
 	}
@@ -703,7 +660,7 @@ ThumbnailPixmapCache::Impl::getThumbFilePath(
 		).toHex()
 	);
 	QString const orig_path_hash_str(
-		QLatin1String(orig_path_hash.data(), orig_path_hash.size())
+		QString::fromAscii(orig_path_hash.data(), orig_path_hash.size())
 	);
 	
 	QFileInfo const orig_img_path(image_id.filePath());
@@ -711,10 +668,10 @@ ThumbnailPixmapCache::Impl::getThumbFilePath(
 	thumb_file_path += QChar('/');
 	thumb_file_path += orig_img_path.baseName();
 	thumb_file_path += QChar('_');
-	thumb_file_path += QString::number(image_id.zeroBasedPage());
+	thumb_file_path += QString::number(image_id.page());
 	thumb_file_path += QChar('_');
 	thumb_file_path += orig_path_hash_str;
-	thumb_file_path += QLatin1String(".png");
+	thumb_file_path += QString::fromAscii(".png");
 	
 	return thumb_file_path;
 }
@@ -733,7 +690,7 @@ ThumbnailPixmapCache::Impl::makeThumbnail(
 	
 	if (image.format() == QImage::Format_Indexed8 && image.isGrayscale()) {
 		// This will be faster than QImage::scale().
-		return scaleToGray(GrayImage(image), to_size);
+		return imageproc::scaleToGray(image, to_size);
 	}
 	
 	return image.scaled(
@@ -835,7 +792,7 @@ ThumbnailPixmapCache::Impl::processLoadResult(LoadResultEvent* result)
 	// Notify listeners.
 	ThumbnailLoadResult const load_result(result->status(), pixmap);
 	typedef boost::weak_ptr<CompletionHandler> WeakHandler;
-	for (WeakHandler const& wh: completion_handlers) {
+	BOOST_FOREACH (WeakHandler const& wh, completion_handlers) {
 		boost::shared_ptr<CompletionHandler> const sh(wh.lock());
 		if (sh.get()) {
 			(*sh)(load_result);
