@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,15 +21,13 @@
 
 #include "NonCopyable.h"
 #include "AbstractFilter.h"
-#include "PageView.h"
 #include "IntrusivePtr.h"
 #include "FilterResult.h"
 #include "SafeDeletingQObjectPtr.h"
-#include "Settings.h"
 
 class PageId;
+class PageLayout;
 class QString;
-class PageSelectionAccessor;
 
 namespace select_content
 {
@@ -49,25 +47,15 @@ class Filter : public AbstractFilter
 {
 	DECLARE_NON_COPYABLE(Filter)
 public:
-	Filter(PageSelectionAccessor const& page_selection_accessor);
+	Filter();
 	
 	virtual ~Filter();
 	
 	virtual QString getName() const;
 	
-	virtual PageView getView() const;
-
-	virtual void performRelinking(AbstractRelinker const& relinker);
-
+	virtual PageSequence::View getView() const;
+	
 	virtual void preUpdateUI(FilterUiInterface* ui, PageId const& page_id);
-
-	virtual void updateStatistics() { m_ptrSettings->updateDeviation(); }
-
-    virtual int selectedPageOrder() const;
-
-    virtual void selectPageOrder(int option);
-
-    virtual std::vector<PageOrderOption> pageOrderOptions() const;
 	
 	virtual QDomElement saveSettings(
 		ProjectWriter const& writer, QDomDocument& doc) const;
@@ -84,8 +72,6 @@ public:
 		IntrusivePtr<select_content::CacheDrivenTask> const& next_task);
 	
 	OptionsWidget* optionsWidget() { return m_ptrOptionsWidget.get(); }
-    Settings* getSettings() { return m_ptrSettings.get(); }
-    void invalidateSetting(PageId const& page_id);
 private:
 	void writePageSettings(
 		QDomDocument& doc, QDomElement& filter_el,
@@ -93,8 +79,6 @@ private:
 	
 	IntrusivePtr<Settings> m_ptrSettings;
 	SafeDeletingQObjectPtr<OptionsWidget> m_ptrOptionsWidget;
-    std::vector<PageOrderOption> m_pageOrderOptions;
-    int m_selectedPageOrder;
 };
 
 } // namespace deskew
