@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 #define DESKEW_THUMBNAIL_H_
 
 #include "ThumbnailBase.h"
-#include "IntrusivePtr.h"
 
 class QSizeF;
 class ThumbnailPixmapCache;
@@ -33,16 +32,13 @@ namespace deskew
 class Thumbnail : public ThumbnailBase
 {
 public:
-	Thumbnail(IntrusivePtr<ThumbnailPixmapCache> const& thumbnail_cache,
-		QSizeF const& max_size, ImageId const& image_id,
-		ImageTransformation const& xform, bool deviant=false);
+	Thumbnail(ThumbnailPixmapCache& thumbnail_cache, QSizeF const& max_size,
+		ImageId const& image_id, ImageTransformation const& xform);
 	
 	virtual void paintOverImage(
 		QPainter& painter,
 		QTransform const& image_to_display,
 		QTransform const& thumb_to_display);
-private:
-	bool m_deviant;
 };
 
 } // namespace deskew
