@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,10 +16,10 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "ImageView.h"
-#include "ImageView.moc"
-#include "ImagePresentation.h"
-#include "OutputMargins.h"
+#include "ImageView.h.moc"
+#include "ImageTransformation.h"
+#include "Dpm.h"
+#include "Dpi.h"
 
 namespace output
 {
@@ -27,18 +27,37 @@ namespace output
 ImageView::ImageView(QImage const& image, QImage const& downscaled_image)
 :	ImageViewBase(
 		image, downscaled_image,
-		ImagePresentation(QTransform(), QRectF(image.rect())),
-		OutputMargins()
-	),
-	m_dragHandler(*this),
-	m_zoomHandler(*this)
+		ImageTransformation(image.rect(), Dpm(image))
+	)
 {
-	rootInteractionHandler().makeLastFollower(m_dragHandler);
-	rootInteractionHandler().makeLastFollower(m_zoomHandler);
 }
 
 ImageView::~ImageView()
 {
 }
 
+void
+ImageView::wheelEvent(QWheelEvent* const event)
+{
+	handleZooming(event);
+}
+
+void
+ImageView::mousePressEvent(QMouseEvent* const event)
+{
+	handleImageDragging(event);
+}
+
+void
+ImageView::mouseReleaseEvent(QMouseEvent* const event)
+{
+	handleImageDragging(event);
+}
+
+void
+ImageView::mouseMoveEvent(QMouseEvent* const event)
+{
+	handleImageDragging(event);
+}
+
 } // namespace output
