@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,11 +21,9 @@
 
 #include "OutputImageParams.h"
 #include "OutputFileParams.h"
-#include "ZoneSet.h"
 
 class QDomDocument;
 class QDomElement;
-class QString;
 
 namespace output
 {
@@ -33,34 +31,19 @@ namespace output
 class OutputParams
 {
 public:
-	OutputParams(OutputImageParams const& output_image_params,
-		OutputFileParams const& output_file_params,
-		OutputFileParams const& automask_file_params,
-		OutputFileParams const& speckles_file_params,
-		ZoneSet const& picture_zones, ZoneSet const& fill_zones);
-
-    explicit OutputParams(QDomElement const& el);
+	OutputParams(OutputImageParams const& image_params,
+		OutputFileParams const& file_params);
+	
+	explicit OutputParams(QDomElement const& el);
 	
 	QDomElement toXml(QDomDocument& doc, QString const& name) const;
 	
-	OutputImageParams const& outputImageParams() const { return m_outputImageParams; }
+	OutputImageParams const& imageParams() const { return m_imageParams; }
 	
-	OutputFileParams const& outputFileParams() const { return m_outputFileParams; }
-
-	OutputFileParams const& automaskFileParams() const { return m_automaskFileParams; }
-
-	OutputFileParams const& specklesFileParams() const { return m_specklesFileParams; }
-
-	ZoneSet const& pictureZones() const { return m_pictureZones; }
-
-	ZoneSet const& fillZones() const { return m_fillZones; }
+	OutputFileParams const& fileParams() const { return m_fileParams; }
 private:
-	OutputImageParams m_outputImageParams;
-	OutputFileParams m_outputFileParams;
-	OutputFileParams m_automaskFileParams;
-	OutputFileParams m_specklesFileParams;
-	ZoneSet m_pictureZones;
-	ZoneSet m_fillZones;
+	OutputImageParams m_imageParams;
+	OutputFileParams m_fileParams;
 };
 
 } // namespace output
