@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,18 +25,13 @@
 #include "Margins.h"
 #include "Alignment.h"
 #include "Params.h"
-#include "ProjectPages.h"
+#include "PageSequence.h"
 #include "ProjectReader.h"
 #include "ProjectWriter.h"
 #include "CacheDrivenTask.h"
-#include "OrderByWidthProvider.h"
-#include "OrderByHeightProvider.h"
-#include "OrderByAlignment.h"
 #include "Utils.h"
-#ifndef Q_MOC_RUN
 #include <boost/lambda/lambda.hpp>
 #include <boost/lambda/bind.hpp>
-#endif
 #include <QRectF>
 #include <QSizeF>
 #include <QString>
@@ -44,41 +39,17 @@
 #include <QCoreApplication>
 #include <QDomDocument>
 #include <QDomElement>
-#include <assert.h>
-#include "CommandLine.h"
-#include "XmlMarshaller.h"
-#include "XmlUnmarshaller.h"
 
 namespace page_layout
 {
 
-Filter::Filter(IntrusivePtr<ProjectPages> const& pages,
+Filter::Filter(IntrusivePtr<PageSequence> const& pages,
 	PageSelectionAccessor const& page_selection_accessor)
-:	m_ptrPages(pages),
-	m_ptrSettings(new Settings),
-	m_selectedPageOrder(0)
+:	m_ptrSettings(new Settings)
 {
-	if (CommandLine::get().isGui()) {
-		m_ptrOptionsWidget.reset(
-			new OptionsWidget(m_ptrSettings, page_selection_accessor)
-		);
-	}
-
-	typedef PageOrderOption::ProviderPtr ProviderPtr;
-
-	ProviderPtr const default_order;
-    ProviderPtr const order_by_alignment(new OrderByAlignment(m_ptrSettings));
-	ProviderPtr const order_by_width(new OrderByWidthProvider(m_ptrSettings));
-	ProviderPtr const order_by_height(new OrderByHeightProvider(m_ptrSettings));
-	m_pageOrderOptions.push_back(PageOrderOption(tr("Natural order"), default_order));
-    m_pageOrderOptions.push_back(PageOrderOption(QObject::tr("Processed then unprocessed"), ProviderPtr(new OrderByReadiness())));
-    m_pageOrderOptions.push_back(PageOrderOption(tr("Order by alignment type"), order_by_alignment));
-    m_pageOrderOptions.push_back(PageOrderOption(tr("Order by increasing width"), order_by_width,
-                                                 tr("Groups the pages by the width of detected content zone\n"
-                                                    "and it's left and right hard margins")));
-    m_pageOrderOptions.push_back(PageOrderOption(tr("Order by increasing height"), order_by_height,
-                                                 tr("Groups the pages by the height of detected content zone\n"
-                                                    "and it's top and bottom hard margins")));
+	m_ptrOptionsWidget.reset(
+		new OptionsWidget(m_ptrSettings, pages, page_selection_accessor)
+	);
 }
 
 Filter::~Filter()
@@ -88,52 +59,27 @@ Filter::~Filter()
 QString
 Filter::getName() const
 {
-    return tr("Page Layout");
+	return QCoreApplication::translate("page_layout::Filter", "Page Layout");
 }
 
-PageView
+PageSequence::View
 Filter::getView() const
 {
-	return PAGE_VIEW;
+	return PageSequence::PAGE_VIEW;
 }
 
 void
-Filter::selected()
+Filter::pagesRemoved(std::vector<PageId> const& pages)
 {
-	m_ptrSettings->removePagesMissingFrom(m_ptrPages->toPageSequence(getView()));
-}
-
-int
-Filter::selectedPageOrder() const
-{
-	return m_selectedPageOrder;
-}
-
-void
-Filter::selectPageOrder(int option)
-{
-	assert((unsigned)option < m_pageOrderOptions.size());
-	m_selectedPageOrder = option;
-}
-
-std::vector<PageOrderOption>
-Filter::pageOrderOptions() const
-{
-	return m_pageOrderOptions;
-}
-
-void
-Filter::performRelinking(AbstractRelinker const& relinker)
-{
-	m_ptrSettings->performRelinking(relinker);
+	m_ptrSettings->removePages(pages);
 }
 
 void
 Filter::preUpdateUI(FilterUiInterface* ui, PageId const& page_id)
 {
-    MarginsWithAuto const margins_mm(m_ptrSettings->getHardMarginsMM(page_id));
+	Margins const margins_mm(m_ptrSettings->getHardMarginsMM(page_id));
 	Alignment const alignment(m_ptrSettings->getPageAlignment(page_id));
-	m_ptrOptionsWidget->preUpdateUI(page_id, margins_mm, alignment);
+	m_ptrOptionsWidget->preUpdateUI(margins_mm, alignment);
 	ui->setOptionsWidget(m_ptrOptionsWidget.get(), ui->KEEP_OWNERSHIP);
 }
 
@@ -143,13 +89,12 @@ Filter::saveSettings(
 {
 	
 	using namespace boost::lambda;
-	    
+	
 	QDomElement filter_el(doc.createElement("page-layout"));
-
 	writer.enumPages(
-		boost::lambda::bind(
+		bind(
 			&Filter::writePageSettings,
-			this, boost::ref(doc), var(filter_el), boost::lambda::_1, boost::lambda::_2
+			this, boost::ref(doc), var(filter_el), _1, _2
 		)
 	);
 	
@@ -161,7 +106,7 @@ Filter::writePageSettings(
 	QDomDocument& doc, QDomElement& filter_el,
 	PageId const& page_id, int numeric_id) const
 {
-	std::unique_ptr<Params> const params(m_ptrSettings->getPageParams(page_id));
+	std::auto_ptr<Params> const params(m_ptrSettings->getPageParams(page_id));
 	if (!params.get()) {
 		return;
 	}
@@ -230,11 +175,12 @@ Filter::invalidateContentBox(PageId const& page_id)
 }
 
 bool
-Filter::checkReadyForOutput(ProjectPages const& pages, PageId const* ignore)
+Filter::checkReadyForOutput(PageSequence const& pages, PageId const* ignore)
 {
- 	if (CommandLine::get().hasDisableCheckOutput())
-  		return true;
-	PageSequence const snapshot(pages.toPageSequence(PAGE_VIEW));
+	PageSequenceSnapshot const snapshot(
+		pages.snapshot(PageSequence::PAGE_VIEW)
+	);
+	
 	return m_ptrSettings->checkEverythingDefined(snapshot, ignore);
 }
 
@@ -260,15 +206,4 @@ Filter::createCacheDrivenTask(
 	);
 }
 
-void
-Filter::invalidateSetting(PageId const& page_id)
-{
-    std::unique_ptr<Params> const params(m_ptrSettings->getPageParams(page_id));
-    if (params.get()) {
-        Params p(*params.get());
-        p.setForceReprocess(Params::RegenerateAll);
-        m_ptrSettings->setPageParams(page_id, p);
-    }
-}
-
 } // namespace page_layout
