@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,56 +21,35 @@
 #include "OptionsWidget.h"
 #include "Task.h"
 #include "Settings.h"
-#include "ProjectPages.h"
+#include "PageSequence.h"
 #include "ProjectReader.h"
 #include "ProjectWriter.h"
 #include "PageId.h"
 #include "ImageId.h"
-#include "PageLayout.h"
 #include "LayoutType.h"
 #include "Params.h"
 #include "CacheDrivenTask.h"
-#include "OrthogonalRotation.h"
-#ifndef Q_MOC_RUN
 #include <boost/lambda/lambda.hpp>
 #include <boost/lambda/bind.hpp>
-#endif
 #include <QString>
 #include <QObject>
 #include <QCoreApplication>
 #include <QDomElement>
 #include <stddef.h>
-#include "CommandLine.h"
-#include "OrderBySplitTypeProvider.h"
-#include "OrderByPageSizeProvider.h"
 
 namespace page_split
 {
 
-Filter::Filter(IntrusivePtr<ProjectPages> const& page_sequence,
+Filter::Filter(IntrusivePtr<PageSequence> const& page_sequence,
 	PageSelectionAccessor const& page_selection_accessor)
 :	m_ptrPages(page_sequence),
-	m_ptrSettings(new Settings),
-	m_selectedPageOrder(0)
+	m_ptrSettings(new Settings)
 {
-	if (CommandLine::get().isGui()) {
-		m_ptrOptionsWidget.reset(
-			new OptionsWidget(
-				m_ptrSettings, m_ptrPages, page_selection_accessor
-			)
-		);
-	}
-	
-	typedef PageOrderOption::ProviderPtr ProviderPtr;
-
-	ProviderPtr const default_order;
-	ProviderPtr const order_by_splitline(new OrderBySplitTypeProvider(m_ptrSettings));
-    ProviderPtr const order_by_page_size(new OrderByPageSizeProvider(m_ptrSettings));
-
-	m_pageOrderOptions.push_back(PageOrderOption(tr("Natural order"), default_order));
-    m_pageOrderOptions.push_back(PageOrderOption(QObject::tr("Processed then unprocessed"), ProviderPtr(new OrderByReadiness())));
-	m_pageOrderOptions.push_back(PageOrderOption(tr("Order by split type"), order_by_splitline));
-    m_pageOrderOptions.push_back(PageOrderOption(tr("Order by page size"), order_by_page_size));
+	m_ptrOptionsWidget.reset(
+		new OptionsWidget(
+			m_ptrSettings, m_ptrPages, page_selection_accessor
+		)
+	);
 }
 
 Filter::~Filter()
@@ -83,22 +62,16 @@ Filter::getName() const
 	return QCoreApplication::translate("page_split::Filter", "Split Pages");
 }
 
-PageView
+PageSequence::View
 Filter::getView() const
 {
-	return IMAGE_VIEW;
-}
-
-void
-Filter::performRelinking(AbstractRelinker const& relinker)
-{
-	m_ptrSettings->performRelinking(relinker);
+	return PageSequence::IMAGE_VIEW;
 }
 
 void
 Filter::preUpdateUI(FilterUiInterface* ui, PageId const& page_id)
 {
-	m_ptrOptionsWidget->preUpdateUI(page_id);
+	m_ptrOptionsWidget->preUpdateUI(page_id.imageId());
 	ui->setOptionsWidget(m_ptrOptionsWidget.get(), ui->KEEP_OWNERSHIP);
 }
 
@@ -115,9 +88,9 @@ Filter::saveSettings(
 	);
 	
 	writer.enumImages(
-		boost::lambda::bind(
+		bind(
 			&Filter::writeImageSettings,
-			this, boost::ref(doc), var(filter_el), boost::lambda::_1, boost::lambda::_2
+			this, boost::ref(doc), var(filter_el), _1, _2
 		)
 	);
 	
@@ -176,26 +149,6 @@ Filter::loadSettings(
 	}
 }
 
-void
-Filter::pageOrientationUpdate(
-	ImageId const& image_id, OrthogonalRotation const& orientation)
-{
-	Settings::Record const record(m_ptrSettings->getPageRecord(image_id));
-
-	if (record.layoutType() && *record.layoutType() != AUTO_LAYOUT_TYPE) {
-		// The layout type was set manually, so we don't care about orientation.
-		return;
-	}
-
-	if (record.params() && record.params()->dependencies().orientation() == orientation) {
-		// We've already estimated the number of pages for this orientation.
-		return;
-	}
-
-	// Use orientation to update the number of logical pages in an image.
-     m_ptrPages->autoSetLayoutTypeFor(image_id, orientation);
-}
-
 void
 Filter::writeImageSettings(
 	QDomDocument& doc, QDomElement& filter_el,
@@ -219,14 +172,14 @@ Filter::writeImageSettings(
 
 IntrusivePtr<Task>
 Filter::createTask(
-	PageInfo const& page_info,
+	PageId const& page_id,
 	IntrusivePtr<deskew::Task> const& next_task,
 	bool const batch_processing, bool const debug)
 {
 	return IntrusivePtr<Task>(
 		new Task(
 			IntrusivePtr<Filter>(this), m_ptrSettings, m_ptrPages,
-			next_task, page_info, batch_processing, debug
+			next_task, page_id.imageId(), batch_processing, debug
 		)
 	);
 }
@@ -236,40 +189,8 @@ Filter::createCacheDrivenTask(
 	IntrusivePtr<deskew::CacheDrivenTask> const& next_task)
 {
 	return IntrusivePtr<CacheDrivenTask>(
-        new CacheDrivenTask(m_ptrSettings, m_ptrPages, next_task)
+		new CacheDrivenTask(m_ptrSettings, next_task)
 	);
 }
 
-std::vector<PageOrderOption>
-Filter::pageOrderOptions() const
-{
-	return m_pageOrderOptions;
-}
-
-int
-Filter::selectedPageOrder() const
-{
-	return m_selectedPageOrder;
-}
-
-void
-Filter::selectPageOrder(int option)
-{
-	assert((unsigned)option < m_pageOrderOptions.size());
-	m_selectedPageOrder = option;
-}
-
-void
-Filter::invalidateSetting(PageId const& page_id)
-{
-    Settings::Record record(m_ptrSettings->getPageRecord(page_id.imageId()));
-    if (record.params()) {
-        Params p(*record.params());
-        p.setForceReprocess(Params::RegenerateAll);
-        Settings::UpdateAction ua;
-        ua.setParams(p);
-        m_ptrSettings->updatePage(page_id.imageId(), ua);
-    }
-}
-
 } // page_split
