@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,20 +21,14 @@
 
 #include "NonCopyable.h"
 #include "AbstractFilter.h"
-#include "PageView.h"
 #include "IntrusivePtr.h"
 #include "FilterResult.h"
 #include "SafeDeletingQObjectPtr.h"
-#include <set>
-#include "PageOrderOption.h"
-#include <QCoreApplication>
 
 class PageId;
 class ImageId;
-class PageInfo;
-class ProjectPages;
+class PageSequence;
 class PageSelectionAccessor;
-class OrthogonalRotation;
 
 namespace deskew
 {
@@ -54,18 +48,15 @@ class Params;
 class Filter : public AbstractFilter
 {
 	DECLARE_NON_COPYABLE(Filter)
-	Q_DECLARE_TR_FUNCTIONS(page_split::Filter)
 public:
-	Filter(IntrusivePtr<ProjectPages> const& page_sequence,
+	Filter(IntrusivePtr<PageSequence> const& page_sequence,
 		PageSelectionAccessor const& page_selection_accessor);
 	
 	virtual ~Filter();
 	
 	virtual QString getName() const;
 	
-	virtual PageView getView() const;
-
-	virtual void performRelinking(AbstractRelinker const& relinker);
+	virtual PageSequence::View getView() const;
 	
 	virtual void preUpdateUI(FilterUiInterface* ui, PageId const& page_id);
 	
@@ -75,7 +66,7 @@ public:
 	virtual void loadSettings(
 		ProjectReader const& reader, QDomElement const& filters_el);
 	
-	IntrusivePtr<Task> createTask(PageInfo const& page_info,
+	IntrusivePtr<Task> createTask(PageId const& page_id,
 		IntrusivePtr<deskew::Task> const& next_task,
 		bool batch_processing, bool debug);
 	
@@ -83,26 +74,14 @@ public:
 		IntrusivePtr<deskew::CacheDrivenTask> const& next_task);
 	
 	OptionsWidget* optionsWidget() { return m_ptrOptionsWidget.get(); }
-
-	void pageOrientationUpdate(
-		ImageId const& image_id, OrthogonalRotation const& orientation);
-
-    Settings* getSettings() { return m_ptrSettings.get(); }
-    void invalidateSetting(PageId const& page_id);
-	
-	virtual std::vector<PageOrderOption> pageOrderOptions() const;
-	virtual int selectedPageOrder() const;
-	virtual void selectPageOrder(int option);
 private:
 	void writeImageSettings(
 		QDomDocument& doc, QDomElement& filter_el,
 		ImageId const& image_id, int const numeric_id) const;
 	
-	IntrusivePtr<ProjectPages> m_ptrPages;
+	IntrusivePtr<PageSequence> m_ptrPages;
 	IntrusivePtr<Settings> m_ptrSettings;
 	SafeDeletingQObjectPtr<OptionsWidget> m_ptrOptionsWidget;
-	std::vector<PageOrderOption> m_pageOrderOptions;
-	int m_selectedPageOrder;
 };
 
 } // namespace page_split
