@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-	Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
+    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 #include "VertLineFinder.h"
 #include "ContentSpanFinder.h"
 #include "ImageMetadata.h"
-#include "ProjectPages.h"
+#include "PageSequence.h"
 #include "DebugImages.h"
 #include "Dpi.h"
 #include "ImageTransformation.h"
@@ -47,6 +47,9 @@
 #include "imageproc/Grayscale.h"
 #include "imageproc/GrayRasterOp.h"
 #include "imageproc/PolygonRasterizer.h"
+#include <boost/foreach.hpp>
+#include <boost/lambda/lambda.hpp>
+#include <boost/lambda/bind.hpp>
 #include <QRect>
 #include <QRectF>
 #include <QSize>
@@ -90,104 +93,266 @@ struct CenterComparator
 	}
 };
 
+void selectHorBordersInPlace(QImage& image)
+{
+	int const w = image.width();
+	int const h = image.height();
+	
+	unsigned char* const image_data = image.bits();
+	int const image_bpl = image.bytesPerLine();
+	
+	std::vector<unsigned char> tmp_line(h, 0x00);
+	
+	for (int x = 0; x < w; ++x) {
+		// Left to right.
+		unsigned char* p_image = image_data + x;
+		unsigned char prev_pixel = 0x00; // Black vertical border.
+		for (int y = 0; y < h; ++y, p_image += image_bpl) {
+			prev_pixel = std::max(*p_image, prev_pixel);
+			tmp_line[y] = prev_pixel;
+		}
+		
+		// Right to left
+		p_image = image_data + x + (h - 1) * image_bpl;
+		prev_pixel = 0x00; // Black vertical border.
+		for (int y = h - 1; y >= 0; --y, p_image -= image_bpl) {
+			prev_pixel = std::max(
+				*p_image,
+				std::min(prev_pixel, tmp_line[y])
+			);
+			*p_image = prev_pixel;
+		}
+	}
+}
+
+QImage removeDarkHorBorders(QImage const& src)
+{
+	QImage dst(src);
+	
+	selectHorBordersInPlace(dst);
+	grayRasterOp<GRopInvert<GRopClippedSubtract<GRopDst, GRopSrc> > >(dst, src);
+	
+	return dst;
+}
+
+QImage detectHorShadows(QImage const& src)
+{
+	QImage long_hor_lines(openGray(src, QSize(100, 1), 0xff));
+	long_hor_lines = removeDarkHorBorders(long_hor_lines);
+	return openGray(long_hor_lines, QSize(100, 1), 0xff);
+}
+
+void countOffcutPixels(
+	unsigned* left_sum, unsigned* right_sum,
+	QLineF const& left_line, QLineF const& right_line,
+	QImage const& gray_downscaled, QImage const& hor_shadows,
+	QTransform const& out_to_downscaled, DebugImages* dbg)
+{
+	assert(left_sum && right_sum);
+	
+	QImage seed(createFramedImage(gray_downscaled.size()));
+	
+	// Remove parts of the vertical seed lines.
+	// We don't want seed pixels to touch text.
+	{
+		int const width = seed.width();
+		int const height = seed.height();
+		uint8_t* seed_line = seed.bits();
+		int const seed_stride = seed.bytesPerLine();
+		int const from = height / 10;
+		int const to = height - from;
+		seed_line += from * seed_stride;
+		for (int y = from; y < to; ++y) {
+			seed_line[0] = 0xff;
+			seed_line[width - 1] = 0xff;
+			seed_line += seed_stride;
+		}
+	}
+	
+	grayRasterOp<GRopDarkest<GRopSrc, GRopDst> >(
+		seed, hor_shadows
+	);
+	
+	seedFillGrayInPlace(seed, gray_downscaled, CONN8);
+	
+	typedef GRopInvert<
+		GRopUnclippedSubtract<GRopDst, GRopSrc>
+	> SubtractBorders;
+	grayRasterOp<SubtractBorders>(seed, gray_downscaled);
+	if (dbg) {
+		dbg->add(seed, "borders_removed");
+	}
+	
+	BinaryImage bin_img(binarizeWolf(seed, QSize(51, 51)));
+	seed = QImage();
+	
+	PageLayout const left_layout(
+		PageLayout(
+			PageLayout::LEFT_PAGE_PLUS_OFFCUT, left_line
+		).transformed(out_to_downscaled)
+	);
+	QPolygonF const left_poly(
+		left_layout.singlePageOutline(
+			gray_downscaled.rect()
+		)
+	);
+	
+	PageLayout const right_layout(
+		PageLayout(
+			PageLayout::RIGHT_PAGE_PLUS_OFFCUT, right_line
+		).transformed(out_to_downscaled)
+	);
+	QPolygonF const right_poly(
+		right_layout.singlePageOutline(
+			gray_downscaled.rect()
+		)
+	);
+	
+	BinaryImage left_offcut(bin_img);
+	PolygonRasterizer::fillExcept(
+		left_offcut, WHITE, left_poly, Qt::WindingFill
+	);
+	*left_sum = left_offcut.countBlackPixels();
+	if (dbg) {
+		dbg->add(left_offcut, "left_offcut");
+	}
+	
+	left_offcut.release();
+	BinaryImage right_offcut(bin_img.release());
+	PolygonRasterizer::fillExcept(
+		right_offcut, WHITE, right_poly, Qt::WindingFill
+	);
+	*right_sum = right_offcut.countBlackPixels();
+	if (dbg) {
+		dbg->add(right_offcut, "right_offcut");
+	}
+}
+
 /**
  * \brief Try to auto-detect a page layout for a single-page configuration.
  *
  * \param layout_type The requested layout type.  The layout type of
  *        SINGLE_PAGE_UNCUT is not handled here.
  * \param ltr_lines Folding line candidates sorted from left to right.
- * \param image_size The dimensions of the page image.
+ * \param image_size The dimentions of the page image.
  * \param hor_shadows A downscaled grayscale image that contains
  *        long enough and not too thin horizontal lines.
  * \param dbg An optional sink for debugging images.
- * \return The page layout detected or a null unique_ptr.
+ * \return The page layout detected or a null auto_ptr.
  */
-std::unique_ptr<PageLayout> autoDetectSinglePageLayout(
+std::auto_ptr<PageLayout> autoDetectSinglePageLayout(
 	LayoutType const layout_type,
 	std::vector<QLineF> const& ltr_lines,
 	QRectF const& virtual_image_rect,
-	GrayImage const& gray_downscaled,
+	QImage const& gray_downscaled,
 	QTransform const& out_to_downscaled, DebugImages* dbg)
 {
+	QImage const hor_shadows(detectHorShadows(gray_downscaled));
+	if (dbg) {
+		dbg->add(hor_shadows, "hor_shadows");
+	}
+	
+	QRect left_area(hor_shadows.rect());
+	left_area.setWidth(std::min(5, hor_shadows.width()));
+	QRect right_area(left_area);
+	right_area.moveRight(hor_shadows.rect().right());
+	
+	BinaryImage const hor_shadows_bin(binarizeMokji(hor_shadows, 20, 30));
+	if (dbg) {
+		dbg->add(hor_shadows_bin, "hor_shadows_bin");
+	}
+	unsigned left_sum = hor_shadows_bin.countBlackPixels(left_area);
+	unsigned right_sum = hor_shadows_bin.countBlackPixels(right_area);
+	
 	double const image_center = virtual_image_rect.center().x();
 	
 	// A loop just to be able to break from it.
-	do {
+	while (left_sum == 0 && right_sum == 0) {
 		// This whole branch (loop) leads to SINGLE_PAGE_UNCUT,
 		// which conflicts with PAGE_PLUS_OFFCUT.
 		if (layout_type == PAGE_PLUS_OFFCUT) {
 			break;
 		}
 		
-		// If we have a single line close to an edge,
-		// Or more than one line, with the first and the last
-		// ones close to an edge, that looks more like
-		// SINGLE_PAGE_CUT layout.
-		if (!ltr_lines.empty()) {
-			QLineF const& first_line = ltr_lines.front();
-			double const line_center = lineCenterX(first_line);
-			if (fabs(image_center - line_center) > 0.65 * image_center) {
-				break;
-			}
-		}
-		if (ltr_lines.size() > 1) {
-			QLineF const& last_line = ltr_lines.back();
-			double const line_center = lineCenterX(last_line);
-			if (fabs(image_center - line_center) > 0.65 * image_center) {
+		// If we have a single single line close to the edge,
+		// then it's unlikely to be SINGLE_PAGE_UNCUT
+		if (ltr_lines.size() == 1) {
+			QLineF const& line = ltr_lines.front();
+			double const line_center = lineCenterX(line);
+			if (fabs(image_center - line_center) > 0.8 * image_center) {
 				break;
 			}
 		}
 		
-		// Return a SINGLE_PAGE_UNCUT layout.
-		return std::unique_ptr<PageLayout>(new PageLayout(virtual_image_rect));
-	} while (false);
+		// Looks like this scan doesn't have a horizontal shadow that
+		// touches the left or the right edge.  This probably means it
+		// doesn't have a split line there as well, and those that
+		// we found are false positives.
+		return std::auto_ptr<PageLayout>(
+			new PageLayout(
+				PageLayout::SINGLE_PAGE_UNCUT, QLineF()
+			)
+		);
+	}
 	
 	if (ltr_lines.empty()) {
 		// Impossible to detect the layout type.
-		return std::unique_ptr<PageLayout>();
-	} else if (ltr_lines.size() > 1) {
-		return std::unique_ptr<PageLayout>(
-			new PageLayout(virtual_image_rect, ltr_lines.front(), ltr_lines.back())
-		);
-	} else {
-		assert(ltr_lines.size() == 1);
+		return std::auto_ptr<PageLayout>();
+	} else if (ltr_lines.size() == 1) {
 		QLineF const& line = ltr_lines.front();
 		double const line_center = lineCenterX(line);
+		PageLayout::Type plt;
 		if (line_center < image_center) {
-			QLineF const right_line(
-				virtual_image_rect.topRight(), virtual_image_rect.bottomRight()
+			plt = PageLayout::RIGHT_PAGE_PLUS_OFFCUT;
+		} else {
+			plt = PageLayout::LEFT_PAGE_PLUS_OFFCUT;
+		}
+		return std::auto_ptr<PageLayout>(new PageLayout(plt, line));
+	} else {
+		if ((left_sum == 0) == (right_sum == 0)) {
+			countOffcutPixels(
+				&left_sum, &right_sum,
+				ltr_lines.front(), ltr_lines.back(),
+				gray_downscaled, hor_shadows,
+				out_to_downscaled, dbg
 			);
-			return std::unique_ptr<PageLayout>(
-				new PageLayout(virtual_image_rect, line, right_line)
+		}
+		if (left_sum > right_sum) {
+			return std::auto_ptr<PageLayout>(
+				new PageLayout(
+					PageLayout::RIGHT_PAGE_PLUS_OFFCUT,
+					ltr_lines.front()
+				)
 			);
 		} else {
-			QLineF const left_line(
-				virtual_image_rect.topLeft(), virtual_image_rect.bottomLeft()
-			);
-			return std::unique_ptr<PageLayout>(
-				new PageLayout(virtual_image_rect, left_line, line)
+			return std::auto_ptr<PageLayout>(
+				new PageLayout(
+					PageLayout::LEFT_PAGE_PLUS_OFFCUT,
+					ltr_lines.back()
+				)
 			);
 		}
 	}
+	
 }
 
 /**
  * \brief Try to auto-detect a page layout for a two-page configuration.
  *
  * \param ltr_lines Folding line candidates sorted from left to right.
- * \param image_size The dimensions of the page image.
- * \return The page layout detected or a null unique_ptr.
+ * \param image_size The dimentions of the page image.
+ * \return The page layout detected or a null auto_ptr.
  */
-std::unique_ptr<PageLayout> autoDetectTwoPageLayout(
+std::auto_ptr<PageLayout> autoDetectTwoPageLayout(
 	std::vector<QLineF> const& ltr_lines,
 	QRectF const& virtual_image_rect)
 {
 	if (ltr_lines.empty()) {
 		// Impossible to detect the page layout.
-		return std::unique_ptr<PageLayout>();
+		return std::auto_ptr<PageLayout>();
 	} else if (ltr_lines.size() == 1) {
-		return std::unique_ptr<PageLayout>(
-			new PageLayout(virtual_image_rect, ltr_lines.front())
+		return std::auto_ptr<PageLayout>(
+			new PageLayout(PageLayout::TWO_PAGES, ltr_lines.front())
 		);
 	}
 	
@@ -195,7 +360,7 @@ std::unique_ptr<PageLayout> autoDetectTwoPageLayout(
 	double const image_center = virtual_image_rect.center().x();
 	double min_distance = std::numeric_limits<double>::max();
 	QLineF const* best_line = 0;
-	for (QLineF const& line: ltr_lines) {
+	BOOST_FOREACH (QLineF const& line, ltr_lines) {
 		double const line_center = lineCenterX(line);
 		double const distance = fabs(line_center - image_center);
 		if (distance < min_distance) {
@@ -204,8 +369,8 @@ std::unique_ptr<PageLayout> autoDetectTwoPageLayout(
 		}
 	}
 	
-	return std::unique_ptr<PageLayout>(
-		new PageLayout(virtual_image_rect, *best_line)
+	return std::auto_ptr<PageLayout>(
+		new PageLayout(PageLayout::TWO_PAGES, *best_line)
 	);
 }
 
@@ -218,7 +383,7 @@ int numPages(LayoutType const layout_type, ImageTransformation const& pre_xform)
 			QSize const image_size(
 				pre_xform.origRect().size().toSize()
 			);
-			num_pages = ProjectPages::adviseNumberOfLogicalPages(
+			num_pages = PageSequence::adviseNumberOfLogicalPages(
 				ImageMetadata(image_size, pre_xform.origDpi()),
 				pre_xform.preRotation()
 			);
@@ -247,10 +412,10 @@ PageLayoutEstimator::estimatePageLayout(
 	DebugImages* const dbg)
 {
 	if (layout_type == SINGLE_PAGE_UNCUT) {
-		return PageLayout(pre_xform.resultingRect());
+		return PageLayout(PageLayout::SINGLE_PAGE_UNCUT, QLineF());
 	}
 	
-	std::unique_ptr<PageLayout> layout(
+	std::auto_ptr<PageLayout> layout(
 		tryCutAtFoldingLine(layout_type, input, pre_xform, dbg)
 	);
 	if (layout.get()) {
@@ -297,17 +462,17 @@ private:
  * \param pre_xform The logical transformation applied to the input image.
  *        The resulting page layout will be in transformed coordinates.
  * \param dbg An optional sink for debugging images.
- * \return The detected page layout, or a null unique_ptr if page layout
+ * \return The detected page layout, or a null auto_ptr if page layout
  *         could not be detected.
  */
-std::unique_ptr<PageLayout>
+std::auto_ptr<PageLayout>
 PageLayoutEstimator::tryCutAtFoldingLine(
 	LayoutType const layout_type, QImage const& input,
 	ImageTransformation const& pre_xform, DebugImages* const dbg)
 {
 	int const num_pages = numPages(layout_type, pre_xform);
 	
-	GrayImage gray_downscaled;
+	QImage gray_downscaled;
 	QTransform out_to_downscaled;
 	
 	int const max_lines = 8;
@@ -458,13 +623,7 @@ PageLayoutEstimator::cutAtWhitespace(
 			left_offcut, right_offcut, dbg
 		)
 	);
-
-	PageLayout transformed_layout(layout.transformed(xform.inverted()));
-
-	// We don't want a skewed outline!
-	transformed_layout.setUncutOutline(pre_xform.resultingRect());
-	
-	return transformed_layout;
+	return layout.transformed(xform.inverted());
 }
 
 /**
@@ -487,6 +646,7 @@ PageLayoutEstimator::cutAtWhitespaceDeskewed150(
 	bool const left_offcut, bool const right_offcut,
 	DebugImages* dbg)
 {
+	using namespace boost::lambda;
 	
 	int const width = input.width();
 	int const height = input.height();
@@ -516,8 +676,8 @@ PageLayoutEstimator::cutAtWhitespaceDeskewed150(
 	span_finder.setMinWhitespaceWidth(8);
 	
 	std::deque<Span> spans;
-	SlicedHistogram hist(cc_img, SlicedHistogram::COLS);	
-    span_finder.find(hist, [&] (Span const& span) { spans.push_back(span); });
+	SlicedHistogram hist(cc_img, SlicedHistogram::COLS);
+	span_finder.find(hist, bind(&std::deque<Span>::push_back, var(spans), _1));
 	
 	if (dbg) {
 		visualizeSpans(*dbg, spans, input, "spans");
@@ -557,11 +717,11 @@ PageLayoutEstimator::to300DpiBinary(
 	scale_xform.scale(xfactor, yfactor);
 	xform *= scale_xform;
 	QSize const new_size(
-		std::max(1, (int)ceil(xfactor * img.width())),
-		std::max(1, (int)ceil(yfactor * img.height()))
+		(int)ceil(xfactor * img.width()),
+		(int)ceil(yfactor * img.height())
 	);
 	
-	GrayImage const new_image(scaleToGray(GrayImage(img), new_size));
+	QImage const new_image(scaleToGray(img, new_size));
 	return BinaryImage(new_image, binary_threshold);
 }
 
@@ -644,7 +804,7 @@ PageLayoutEstimator::visualizeSpans(
 	{
 		QPainter painter(&spans_img);
 		QBrush const brush(QColor(0xff, 0x00, 0x00, 0x50));
-		for (Span const& span: spans) {
+		BOOST_FOREACH(Span const& span, spans) {
 			QRect const rect(span.begin(), 0, span.width(), height);
 			painter.fillRect(rect, brush);
 		}
@@ -709,9 +869,7 @@ PageLayoutEstimator::processContentSpansSinglePage(
 {
 	assert(layout_type == AUTO_LAYOUT_TYPE
 			|| layout_type == PAGE_PLUS_OFFCUT);
-
-	QRectF const virtual_image_rect(0, 0, width, height);
-
+	
 	// Just to be able to break from it.
 	while (left_offcut && !right_offcut
 			&& layout_type == AUTO_LAYOUT_TYPE) {
@@ -731,11 +889,7 @@ PageLayoutEstimator::processContentSpansSinglePage(
 				x = std::min(spans[0].end() + 20, width);
 			}
 		}
-
-		QLineF const right_line(
-			virtual_image_rect.topRight(), virtual_image_rect.bottomRight()
-		);
-		return PageLayout(virtual_image_rect, vertLine(x), right_line);
+		return PageLayout::rightPagePlusOffcut(vertLine(x));
 	}
 	
 	// Just to be able to break from it.
@@ -757,20 +911,20 @@ PageLayoutEstimator::processContentSpansSinglePage(
 				x = std::max(spans.back().begin() - 20, 0);
 			}
 		}
-
-		QLineF const left_line(
-			virtual_image_rect.topLeft(), virtual_image_rect.bottomLeft()
-		);
-		return PageLayout(virtual_image_rect, left_line, vertLine(x));
+		return PageLayout::leftPagePlusOffcut(vertLine(x));
 	}
 	
-	if (layout_type == PAGE_PLUS_OFFCUT) {
-		QLineF const line1(virtual_image_rect.topLeft(), virtual_image_rect.bottomLeft());
-		QLineF const line2(virtual_image_rect.topRight(), virtual_image_rect.bottomRight());
-		return PageLayout(virtual_image_rect, line1, line2);
+	if (spans.empty()) {
+		return PageLayout::singlePageUncut();
 	} else {
-		// Returning a SINGLE_PAGE_UNCUT layout.
-		return PageLayout(virtual_image_rect);
+		// If there is more whitespace before the first content
+		// span than after the last one, cut on the left,
+		// otherwise cut on the right.
+		if (spans.front().begin() < width - spans.back().end()) {
+			return PageLayout::rightPagePlusOffcut(vertLine(0.0));
+		} else {
+			return PageLayout::leftPagePlusOffcut(vertLine(width));
+		}
 	}
 }
 
@@ -780,14 +934,12 @@ PageLayoutEstimator::processContentSpansTwoPages(
 	std::deque<Span> const& spans, int const width, int const height)
 {
 	assert(layout_type == AUTO_LAYOUT_TYPE || layout_type == TWO_PAGES);
-
-	QRectF const virtual_image_rect(0, 0, width, height);
-
+	
 	double x;
 	if (spans.empty()) {
 		x = 0.5 * width;
 	} else if (spans.size() == 1) {
-		return processTwoPagesWithSingleSpan(spans.front(), width, height);
+		return processTwoPagesWithSingleSpan(spans.front(), width);
 	} else {
 		// GapInfo.first: the amount of content preceding this gap.
 		// GapInfo.second: the amount of content following this gap.
@@ -829,7 +981,7 @@ PageLayoutEstimator::processContentSpansTwoPages(
 		if (best_ratio < 0.25) {
 			// Probably one of the pages is just empty.
 			return processTwoPagesWithSingleSpan(
-				Span(content_begin, content_end), width, height
+				Span(content_begin, content_end), width
 			);
 		}
 		
@@ -867,14 +1019,12 @@ PageLayoutEstimator::processContentSpansTwoPages(
 		Span const gap(spans[widest_gap],  spans[widest_gap + 1]);
 		x = gap.center();
 	}
-	return PageLayout(virtual_image_rect, vertLine(x));
+	return PageLayout(PageLayout::TWO_PAGES, vertLine(x));
 }
 
 PageLayout
-PageLayoutEstimator::processTwoPagesWithSingleSpan(Span const& span, int width, int height)
+PageLayoutEstimator::processTwoPagesWithSingleSpan(Span const& span, int width)
 {
-	QRectF const virtual_image_rect(0, 0, width, height);
-
 	double const page_center = 0.5 * width;
 	double const box_center = span.center();
 	double const box_half_width = 0.5 * span.width();
@@ -895,7 +1045,7 @@ PageLayoutEstimator::processTwoPagesWithSingleSpan(Span const& span, int width,
 		}
 	}
 	
-	return PageLayout(virtual_image_rect, vertLine(x));
+	return PageLayout(PageLayout::TWO_PAGES, vertLine(x));
 }
 
 QLineF
